/*
eeshlo wrote a texture plugin for sunsky for blender, released the binary for 1 version of blender/python, then disappeared.  Asshole.

Written by Joseph Miller joseph c miller 2 at that gmail place
Help with coords provided by subcomandante at yafaray IRC

*      This library is free software; you can redistribute it and/or
 *      modify it under the terms of the GNU Lesser General Public
 *      License as published by the Free Software Foundation; either
 *      version 2.1 of the License, or (at your option) any later version.
 *
 *      This library is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *      Lesser General Public License for more details.
 *
 *      You should have received a copy of the GNU Lesser General Public
 *      License along with this library; if not, write to the Free Software
 *      Foundation,Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

*/

/*
 * Copyright (c) 2000, Not a Number b.v.
 * This code is based at Ken Perlin's famous works...
 *
 */

#include "plugin.h"
#include <stdio.h>

// see sunsky.cc for source
extern void sunsky_get_rgb(float *red, float *green, float *blue, float sun_x_dir, float sun_y_dir, float sun_z_dir, float turb, float x_coord, float y_coord, float z_coord, float sun_intensity);

/*
 * Copyright (c) 1999, Not a Number / NeoGeo b.v.
 * Adapted by :Predrag kurtovic 2.05.2000
 */

/* ******************** GLOBAL VARIABLES ***************** */

/* Texture name */

char name[24]= "SunskyGPL";

/* Subtype names must be less than 15 characters */

#define NR_TYPES	1
char stnames[NR_TYPES][16]= {"Default"};

/* Structure for buttons,
 *  butcode      name           default  min  max  0
 */

VarStruct varstr[]= {
   {  LABEL,   "SunskyGPL!",0, 0, 0, "Now GPL version released" },
  {NUMSLI|FLO,"Turbidity",     4., 0., 12., "3-5 is normal day"},
  //{NUMSLI|FLO,"Horz Bright", 1., -1., 1., "0.9-1.5 on normal day"},
  //{NUMSLI|FLO,"Sun Width", 15., 0., 50., "width of solar region"},
  {NUMSLI|FLO,"Sun Intens.",     0.6, 0., 40., "Glow intensity"},
  //{NUMSLI|FLO,"Backscatter",     0., -5., 5., "Backscatter only used <0"},
  {NUMSLI|FLO,"x=",     0.1, -1., 1., "Sun x position"},
  {NUMSLI|FLO,"y=",     0.2, -1., 1., "Sun y position"},
  {NUMSLI|FLO,"z=", 0.1, -1., 1., "Sun z position"}
};

/* The cast struct is for input in the main doit function
   Varstr and Cast must have the same variables in the same order,
   INCLUDING dummy variables for label fields. */

typedef struct Cast {
	int labelvar1;
	float turbidity;
	//float horzbright;
	//float sundiskw;
	float sundiski;
	//float backscatter;
	float xcoord;
	float ycoord;
	float zcoord;
} Cast;

/* result:
   Intensity, R, G, B, Alpha, nor.x, nor.y, nor.z
 */

float result[8];

/* cfra: the current frame */

float cfra;

int plugin_tex_doit(int, Cast*, float*, float*, float*);

/* ******************** Fixed functions ***************** */

int plugin_tex_getversion(void)
{
	return B_PLUGIN_VERSION;
}

void plugin_but_changed(int but)
{
}

void plugin_init(void)
{

}

/* this function should not be changed: */

void plugin_getinfo(PluginInfo *info)
{
	info->name= name;
	info->stypes= NR_TYPES;
	info->nvars= sizeof(varstr)/sizeof(VarStruct);

	info->snames= stnames[0];
	info->result= result;
	info->cfra= &cfra;
	info->varstr= varstr;

	info->init= plugin_init;
	info->tex_doit=  (TexDoit) plugin_tex_doit;
	info->callback= plugin_but_changed;
}

/* ********************* the texture ******************** */


/* return 0: One channel texture
   return 1: RGB texture
   return 2: Normals texture */

int plugin_tex_doit(int stype, Cast *cast, float *texvec, float *dxt, float *dyt)
{

	float red, green, blue, x_coord, y_coord, z_coord, dist;

	/* always return this value */
	result[0] = 0.;
	x_coord = texvec[0];
	y_coord = texvec[1];
	z_coord = texvec[2];
	dist = sqrt(x_coord*x_coord + y_coord*y_coord);

	if ( dist <= 1 ) {
		/*
		texvec[0] is x in range [-1,1]
		texvec[1] is y in range [-1,1]
		texvec[2] is z in range [-1,1]
		*/

		// fix this!
		z_coord = sqrt(1-dist);

		//fprintf(stderr, "d = %f\n", sqrt(texvec[0]*texvec[0] + texvec[1]*texvec[1]));

		// see sunsky.cc near end of file
		// swapped y and z to change to camera view coordinates
		sunsky_get_rgb(&red, &green, &blue, cast->xcoord, cast->ycoord, cast->zcoord, cast->turbidity, x_coord, z_coord, y_coord, cast->sundiski);

		/* color? then return 1;
		 *
		 * this is r, g, b, a:
		 */
		
		result[1] = red;
		result[2] = green;
		result[3] = blue;
		result[4] = 1.0;
	} else {
		result[1] = 0.0;
		result[2] = 0.0;
		result[3] = 0.0;
		result[4] = 0.0;
		//fprintf(stderr, "x = %f, y = %f, d = %f\n", texvec[0], texvec[1], sqrt(texvec[0]*texvec[0] + texvec[1]*texvec[1]));
	}
	//result[4]= 1.0;

	return 1;
}

